%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CM1_DualPol.m
%
% This script will read in CM1 output files and calculate the dual-pol
% variables from rain and melting hail. These will then be saved in .mat
% files for later analysis.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear; clc;

main_path = '/graupel/s0/swl5295/PrecipBlob/CM1/cm1r19/run/CM1_SizeSorting/';
case_path = 'ElReno/';
time_path = 'Mature/';
path = [main_path case_path time_path];

first_file = uigetfile('*.nc', 'Select any file');    % Select a file to get attributes of model run
z = ncread([path first_file],'lev');                  % AKA things that won't change in time
y = ncread([path first_file],'lat');
x = ncread([path first_file],'lon');

% Setup meshgrid for plotting
[X,Y] = meshgrid(x,y);

% Bring in radar and custom colormaps
CM1colors;
dualpol_colmap;
color_test;

[filen] = uigetfile('*.nc', 'Select CM1 files', 'MultiSelect','on');

num_times = length(filen);

%%%% Calculate some constants that won't change in time %%%%
HailMaxD = 40.0;  %max hail size allowed in mm (keep to one decimal place)

%DEFINE SOME CONSTANTS
rhoa = 1.2;  %Air density (kg/m3)
rhol = 1000; %Liquid water density (kg/m3)
rhoi = 917;  %Density of solid ice (kg/m3)
mu = 0;      %shape factor for DSD in morrison 2-moment scheme. 0 in Hughbert's scheme (personal communication)
Dmin = 0.05; %minimum diameter in mm
Dmax = 7.95; %maximum diameter in mm
dD = 0.1;    %diameter increment in mm
eps = 77.9210 -13.1820i;   %S band 20 degrees
kw=abs((eps-1)./(eps+2));
lam=109.7;  %s band wavelength in mm

%Create array of D for discretizing DSD
D = (Dmin:dD:Dmax);

%Load in T-matrix stuff
load('water_0c_sband_80bins.out');  %load in T-matrix amplitudes
fh = (water_0c_sband_80bins(:,3) - sqrt(-1) .* water_0c_sband_80bins(:,4)) .* 1000.0 ;  %fh in mm
fv = -1.0 .* (water_0c_sband_80bins(:,1) + sqrt(-1) .* water_0c_sband_80bins(:,2)) .* 1000.0 ;  %fv in mm


% %==========================
% %HAIL STUFF - take only the stones up to 40.0 cm
load NewWetSband.out; tmatr = NewWetSband(1:(HailMaxD*10),:); eps_w = 81.0655 -23.2853i; K_w = (eps_w - 1.0)./(eps_w + 2.0);
WetHailParam
% disp('Running Wet Hail Parameterization')
%Parse out data
fhhail = complex(tmatr(:,3), -1.*tmatr(:,4));
fvhail = complex(-1.0 .* tmatr(:,1), tmatr(:,2));
ffhhail = complex(tmatr(:,7), tmatr(:,8));
ffvhail = complex(tmatr(:,5), tmatr(:,6));
fbhail = 1000.0.*fhhail'; fahail = 1e3 .* fvhail'; ffbhail = 1e3 .* ffhhail'; ffahail = 1e3 .* ffvhail';
sig = sigR11(1:(HailMaxD*10));  %Only want the small stuff    ***SWITCH BETWEEN R11 and MK ****
sigma=sig.*pi./180;
r=exp(-2.*sigma.^2);
A1=0.25.*(1+r).^2; A2=0.25.*(1-r.^2); A3=(0.375 + 0.5 .* r + 0.125 .* r .^ 4) .^ 2;
A4=(0.375 - 0.5 .* r + 0.125 .* r .^ 4) .* (0.375 + 0.5 .* r + 0.125 .* r .^ 4);
A5=0.125 .* (0.375 + 0.5 .* r + 0.125 .* r .^ 4) .* (1 - r .^ 4);
A7 = 0.5 .* r .* (1+r);
A9 = 0.5 .* r .* (1+r);

Dpmm = Dpart(1:(HailMaxD*10)).*1000;  %particle diameter in mm
nbins = numel(Dpmm);             %number of particle bins for discretization
dDh = gradient(Dpmm);

%Max and Min particle sizes
Dhmax = HailMaxD; Dhmin = min(Dice).*1000;

%%% Pre-allocate some dual-pol arrays %%%
ZH_array = zeros([size(X),num_times]);    % Empty arrays (x by y by time)
ZHr_array = ZH_array;    ZHh_array = ZH_array;
ZDR_array = ZH_array;    
ZDRr_array = ZH_array;   ZDRh_array = ZH_array;
KDP_array = ZH_array;    
KDPr_array = ZH_array;   KDPh_array = ZH_array;
RHV_array = ZH_array;    
RHVr_array = ZH_array;   RHVh_array = ZH_array;

time = zeros(1,num_times);

for i = 1:num_times
    
    temp_file = char(filen(i));
    
    %====================
    %Read in model data
    %===================
    % disp('Reading Model Data')
    time(i) = ncread([path temp_file],'time');
    time = time*60 + 60;   % For "mature"
        
    qr = double(ncread([path temp_file],'qr'));  %Mass mixing ratio in kg/kg
    ncr = double(ncread([path temp_file],'ncr')); %number mixing ratio in /kg

    qh = double(ncread([path temp_file],'qg'));  %Mass mixing ratio in kg/kg
    nch = double(ncread([path temp_file],'ncg')); %number mixing ratio /kg

    % Change dimensions of arrays
    qr = permute(qr,[2 1 3]);
    ncr = permute(ncr,[2 1 3]);
    qh = permute(qh,[2 1 3]);
    nch = permute(nch,[2 1 3]);
    
    %Some model parameters
    si = size(qr);
    nx = si(1);
    ny = si(2);
    ndrops = numel(D);
    nhails = numel(Dpmm);

    %Focusing only on low levels
    NCR = ncr(:,:,1); clear ncr; %just grab surface value
    QR = qr(:,:,1);  clear qr;%just grab surface value
    NCH = nch(:,:,1); clear nch; %just grab surface value
    QH = qh(:,:,1);  clear qh;%just grab surface value

    %disp('Computing PSD parameters')
    %First pass is using the complete gamma to get estimate/prior for lambda
    lambda_r = (pi./6 .* rhol .* NCR./QR .* gamma(mu+4)./gamma(mu+1)).^(1.0./3.0) ./ 1000;  %in mm
    n0_r = NCR .* rhoa .* lambda_r.^(mu+1) ./ gamma(mu+1);

    lambda_h = (pi./6 .* rhoi .* NCH./QH .* gamma(mu+4)./gamma(mu+1)).^(1.0./3.0) ./ 1000;  %in mm
    n0_h = NCH .* rhoa .* lambda_h.^(mu+1) ./ gamma(mu+1);

    %Then we use the incomplete gamma. It's a bit of a kluge, but should be
    %fine outside of the edges of the echo.
    LAMBDA_R = (pi./6 .* rhol .* NCR./QR .* gamma(mu + 4) .* (1-gammainc(lambda_r.*Dmax,mu+4,'upper')-gammainc(lambda_r.*Dmin,mu+4,'lower')) ./...
        gamma(mu + 1) .* (1-gammainc(lambda_r.*Dmax,mu+1,'upper')-gammainc(lambda_r.*Dmin,mu+1,'lower'))).^(1.0./3.0) ./ 1000;  %in mm
    N0_R =  NCR .* rhoa .* LAMBDA_R.^(mu+1) ./ (gamma(mu+1).* (1-gammainc(lambda_r.*Dmax,mu+1,'upper')-gammainc(lambda_r.*Dmin,mu+1,'lower')));

    LAMBDA_H = (pi./6 .* rhoi .* NCH./QH .* gamma(mu + 4) .* (1-gammainc(lambda_h.*Dhmax,mu+4,'upper')-gammainc(lambda_h.*Dhmin,mu+4,'lower')) ./...
        gamma(mu + 1) .* (1-gammainc(lambda_h.*Dhmax,mu+1,'upper')-gammainc(lambda_h.*Dhmin,mu+1,'lower'))).^(1.0./3.0) ./ 1000;  %in mm
    N0_H =  NCH .* rhoa .* LAMBDA_H.^(mu+1) ./ (gamma(mu+1).* (1-gammainc(lambda_h.*Dhmax,mu+1,'upper')-gammainc(lambda_h.*Dhmin,mu+1,'lower')));

    ND = zeros(ndrops,1); NH = zeros(1,nhails);
    ZH_grid = zeros(nx,ny); ZDR_grid = ZH_grid; KDP_grid = ZH_grid; RHV_grid = ZH_grid;
    ZHr_grid = ZH_grid;  ZHh_grid = ZH_grid;
    ZDRr_grid = ZH_grid;  ZDRh_grid = ZH_grid;
    KDPr_grid = ZH_grid;  KDPh_grid = ZH_grid;
    RHVr_grid = ZH_grid;  RHVh_grid = ZH_grid;
    
    %%% Loop over model domain
    for xx=1:nx    %Model x dimension
        for yy=1:ny   %model y dimension

        ND(:) = N0_R(xx,yy) .* D(:).^mu .* exp(-LAMBDA_R(xx,yy) .* D(:));
        NH(:) = N0_H(xx,yy) .* Dpmm(:).^mu .* exp(-LAMBDA_H(xx,yy) .* Dpmm(:));

        [ZHg, ZHg_r, ZHg_h, ZDRg, ZDRg_r, ZDRg_h, KDPg, KDPg_r, KDPg_h, RHVg, RHVg_r, RHVg_h, ZDR_hail_DSD] = ...
            calc_dualpol(ND,fh,fv, NH, HailMaxD, Dpart, fbhail, fahail, ffbhail, ffahail, sigMK);
        ZH_grid(xx,yy) = ZHg;       ZHr_grid(xx,yy) = ZHg_r;    ZHh_grid(xx,yy) = ZHg_h;    
        ZDR_grid(xx,yy) = ZDRg;     ZDRr_grid(xx,yy) = ZDRg_r;  ZDRh_grid(xx,yy) = ZDRg_h;
        KDP_grid(xx,yy) = KDPg;     KDPr_grid(xx,yy) = KDPg_r;  KDPh_grid(xx,yy) = KDPg_h;
        RHV_grid(xx,yy) = RHVg;     RHVr_grid(xx,yy) = RHVg_r;  RHVh_grid(xx,yy) = RHVg_h;

        end
    end

    ZH_array(:,:,i) = ZH_grid;       ZHr_array(:,:,i) = ZHr_grid;    ZHh_array(:,:,i) = ZHh_grid;    
    ZDR_array(:,:,i) = ZDR_grid;     ZDRr_array(:,:,i) = ZDRr_grid;  ZDRh_array(:,:,i) = ZDRh_grid;
    KDP_array(:,:,i) = KDP_grid;     KDPr_array(:,:,i) = KDPr_grid;  KDPh_array(:,:,i) = KDPh_grid;
    RHV_array(:,:,i) = RHV_grid;     RHVr_array(:,:,i) = RHVr_grid;  RHVh_array(:,:,i) = RHVh_grid;
    
%     figure()
%     pcolor(X,Y,squeeze(ZH_array(:,:,i)))
%     shading flat
%     colorbar
    
end



